import os
from pyglet.gl import *
import math

def old_screen_to_world(x, y):
    '''Translate screen (mouse) coordinates to game world coordinates
    '''
    world_x, world_y, world_z = GLdouble(), GLdouble(), GLdouble()
    modelview , projection = (GLdouble * 16)(), (GLdouble * 16)()
    view = (GLint * 4)()
    glGetDoublev(GL_MODELVIEW_MATRIX,modelview)
    glGetDoublev(GL_PROJECTION_MATRIX,projection)
    glGetIntegerv(GL_VIEWPORT,view)
    wx, wy, wz = GLdouble(), GLdouble(), GLdouble()
    # Use point at center of game world
    gluProject(0.0, 0.0, 0.0, modelview, projection, view, wx, wy, wz)
    gluUnProject(x, y, wz, modelview,
                 projection, view, world_x, world_y, world_z)
    return int(world_x.value),int(world_y.value)


class Hashtable(object):
    pass


# Get full path for images, takes an image filename
def imagepath(imagename):
    (head, tail) = os.path.split(__file__)
    return(os.path.join(os.path.split(head)[0], 'data', 'gfx', imagename))


# Get full path for configuration files, takes a configuration filename
def configpath(configname):
    (head, tail) = os.path.split(__file__)
    return(os.path.join(os.path.split(head)[0], 'data', 'cfg', configname))


# Get full path for sound files, takes a sound filename
def soundpath(soundname):
    (head, tail) = os.path.split(__file__)
    return(os.path.join(os.path.split(head)[0], 'data', 'snd', soundname))


def screen_to_world(x, y):
    '''Translate screen (mouse) coordinates to game world coordinates
    '''
    world_x, world_y, world_z = GLdouble(), GLdouble(), GLdouble()
    modelview , projection = (GLdouble * 16)(), (GLdouble * 16)()
    view = (GLint * 4)()
    glGetDoublev(GL_MODELVIEW_MATRIX,modelview)
    glGetDoublev(GL_PROJECTION_MATRIX,projection)
    glGetIntegerv(GL_VIEWPORT,view)
    wx, wy, wz = GLdouble(), GLdouble(), GLdouble()
    gluProject(0.0, 0.0, 0.0, modelview, projection, view, wx, wy, wz)
    # Get mouse point on near and far clipping plane
    wmx1, wmy1, wmz1 = GLdouble(), GLdouble(), GLdouble()
    wmx2, wmy2, wmz2 = GLdouble(), GLdouble(), GLdouble()
    gluUnProject(x, y, 0.0, modelview, projection, view, wmx1, wmy1, wmz1)
    gluUnProject(x, y, 1.0, modelview, projection, view, wmx2, wmy2, wmz2)
    # ray magnitude
    rx = wmx1.value - wmx2.value
    ry = wmy1.value - wmy2.value
    rz = wmz1.value - wmz2.value
    rmag = math.sqrt(rx*rx + ry*ry + rz*rz)
    if rmag > 0.0:
        rx = rx / rmag
        ry = ry / rmag
        rz = rz / rmag
    else:
        return 0.0, 0.0, 0.0
    
    #print "near clip: %s, %s, %s" % (str(wmx1), str(wmy1), str(wmz1))
    #print "far clip: %s, %s, %s" % (str(wmx2), str(wmy2), str(wmz2))
    #print "ray mag: " + str(rmag)
    # Normal (direction) vector
    #print "normal: %f, %f, %f   mag: %f" % (rx, ry, rz, \
    #                                        math.sqrt(rx*rx + ry*ry + rz*rz))

    t = -wmz1.value / rz
    px = wmx1.value + t * rx
    py = wmy1.value + t * ry
    pz = wmz1.value + t * rz

    #print "world mark: (%f) %f, %f, %f" % (t, -px, -py, -pz)
    
    return (px, py, pz)


# Primitives drawing routines

def draw_box(x1, y1, x2, y2):
    glDisable(GL_TEXTURE_2D)
    glLineWidth(1.0)
    glColor3f(1.0, 0.0, 0.0)
    glBegin(GL_LINE_LOOP)
    glVertex3f(x1, y1, 0.0)
    glVertex3f(x2, y1, 0.0)
    glVertex3f(x2, y2, 0.0)
    glVertex3f(x1, y2, 0.0)
    glEnd()
    glColor3f(1.0, 1.0, 1.0)


# Checks to see if two boxes overlap (AABB)
def boxes_overlap(a, b):
#    print repr(a), repr(b)
    return(a[2] > b[0] and a[0] < b[2] and a[1] < b[3] and a[3] > b[1])

