"""Movable object"""

import mobsprite
import math
import misc

class Mob(object):
    DEBUG = False;
    
    maxvel = 400.0   # pixels per second
    close_enough = 20.0   # slop
    box_height = 10.0   # vertical offset for collision box, both sides
    box_width = -10.0  # horizonal offset for collision box
    max_step = 20.0   # usually 2x the box size
    
    def __init__(self, g, x = 0, y = 0, type = 'george'):
        self.g, self.x, self.y, self.type = g, x, y, type
        self.sprite = mobsprite.Mobsprite(self.g, self.x, self.y, self.type)
        self.kind = 'generic'
        self.moving = False
        self.vel = 0.0
        self.dir = 0.0   # Direction is in radians
        self.target_x, self.target_y = 0.0, 0.0
        self.facing = 'right'
        self.active = True


    def __setattr__(self, name, value):
        object.__setattr__(self, name, value)
        if name == 'x' or name == 'y':
            try: self.sprite.set_position(self.x, self.y)
            except: pass
        

    def simstep(self):
        if self.moving == False:
            return
        fullstep = self.vel * self.g.dt
        done = False
        while((not done) and self.moving):
            if fullstep > Mob.max_step:
                step = Mob.max_step
                fullstep -= Mob.max_step
            else:
                step = fullstep
                done = True
            step_x = math.cos(self.dir) * step
            step_y = math.sin(self.dir) * step

            # naive collision detection, check against every mob
            bb = self.get_collbox()
            new_bb = [bb[0] + step_x, bb[1] + step_y, bb[2] + step_x,
                      bb[3] + step_y]
            for mob in self.g.scene.moblist:
                if mob == self:
                    continue

                # hack: let projectiles pass thru their owners
                if self.kind == 'projectile':
                    if self.owner == mob:
                        continue
                # ...and let owners pass thru their projectiles
                if mob.kind == 'projectile':
                    if mob.owner == self:
                        continue
                # and projectiles pass thru each other
                if self.kind == 'projectile' and mob.kind == 'projectile':
                    continue
                    
                if misc.boxes_overlap(new_bb, mob.get_collbox()):
                    if self.kind == 'projectile':
                        mob.collision(self)
                        self.collision(mob)
                    else:
                        self.collision(mob)
                        mob.collision(self)

            if self.moving == True:
                self.x += step_x
                self.y += step_y
                if self.kind == 'enemy' and self.y < 0:
                    self.g.gamestate = 'loserscreen_init'
                if abs(self.target_x - self.x) <= abs(step_x) and \
                       abs(self.target_y - self.y) <= abs(step_y):
                    self.x, self.y = self.target_x, self.target_y
                    self.moving = False
                    self.active = False
                    if self.kind == 'projectile':
                        self.stop_moving(0)
        

    # Starts moving mob to the given x,y position
    def move_to(self, target_x, target_y):
        self.moving = True
        self.target_x, self.target_y = target_x, target_y
        self.vel = Mob.maxvel
        self.dir = math.atan2(target_y - self.y, target_x - self.x)
        if self.target_x < self.x:
            self.facing = 'left'
        else:
            self.facing = 'right'
        

    def draw(self):
        self.sprite.draw(self.facing)
        #self.draw_collhull()


    # Draw the collision hull, for debugging
    def draw_collhull(self):
        x1, y1, x2, y2 = self.get_collbox()
        misc.draw_box(x1, y1, x2, y2)

        
    # Get the points describing the collision box
    def get_collbox(self):
        width = Mob.box_width
        if self.type == 'tree':
            width = -90.0
        return(self.x - self.sprite.image.width / 2.0 - width,
               self.y - Mob.box_height,
               self.x + self.sprite.image.width / 2.0 + width,
               self.y + Mob.box_height)


    # Handle collisions
    def collision(self, mob):
        self.moving = False
        if mob.DEBUG:
            print "Collision stub (%s hit %s)" % (self.type, mob.type)

