"""projectiles"""

import mob
import random
import pyglet
import math
import mobsprite

class Projectile(mob.Mob):
    maxvel = 800.0
    maxbottles = 30
    known_types = ['egg', 'pillbottle1', 'baseball', 'cane', 'dog',
                   'false_teeth', 'slipper', 'pie']

    def __init__(self, g, owner, x = 0, y = 0, type = 'random'):
        if type == 'random':
            type = self.pick_and_play(g)
        super(Projectile, self).__init__(g, owner.x, owner.y, type)
        self.kind = 'projectile'
        self.owner = owner
        self.move_to(x, y)
        self.vel = Projectile.maxvel
        self.active = True
        self.last_hit = None
        self.trim_moblist()

    def collision(self, mob):
        if self.moving == False:
            return
        # stop if we hit anything else after bouncing
        if self.active == False:
            if mob != self.last_hit:
                self.stop_moving(0)
        else:
            self.active = False
            self.last_hit = mob
            # bounce!
            vel = self.vel * (0.2 + random.random() * 0.1)
            dir = self.dir + 3 * math.pi - 0.8 + random.random() * 1.6
            dir %= 2 * math.pi
            t = 0.2 + random.random() * 0.2
            newx = self.x + math.cos(dir) * vel * t
            newy = self.y + math.sin(dir) * vel * t
            self.move_to(newx, newy)
            pyglet.clock.schedule_once(self.stop_moving, t)
            

    def stop_moving(self, dt):
        if self.type == 'egg':
            self.g.sounddict['eggsplat'].play()
            self.splat()
        self.moving = False
        self.vel = 0.0

        
    def trim_moblist(self):
        count = 0
        for mob in self.g.scene.moblist:
            if mob.kind == 'projectile':
                if count == 0:
                    first = mob
                count += 1
        if count > Projectile.maxbottles:
            self.g.scene.moblist.remove(first)
            

    # Pick a random type and play a wacky sound if appropriate
    def pick_and_play(self, g):
        type = Projectile.known_types[int(len(Projectile.known_types) * \
                                      random.random())]
        if type == 'dog':
            g.sounddict['yiff'].play()
        return type

    
    def splat(self):
        if random.random() > 0.5:
            splat = 'eggsplat1'
        else:
            splat = 'eggsplat2'
        x, y = self.sprite._x, self.sprite._y
        self.sprite = mobsprite.Mobsprite(self.g, x, y, splat)
        
