"""Score, time, etc"""

from pyglet.gl import *
import misc

class Scoreboard(object):
    sec_per_clock = 10.0

    def __init__(self, g):
        self.g = g
        self.scoreboardimg = g.imagedict['scoreboard']
        self.scoreboardtex = self.scoreboardimg.get_texture()
        self.clockimg = g.imagedict['clock']
        self.clocktex = self.clockimg.get_texture()
        self.enemyheadimg = g.imagedict['eggboy_head']
        self.enemyheadtex = self.enemyheadimg.get_texture()
        self.last_numclocks = int( \
            self.g.scene.timeleft / Scoreboard.sec_per_clock) - 1


    def draw(self):
        glLoadIdentity()
        glTranslatef(0.0, 0.0, -500.0)
        x1, y1 = misc.old_screen_to_world(0.0, 0.0)
        x2, y2 = misc.old_screen_to_world(self.g.win.width,
                                          self.g.win.height)
        glEnable(self.scoreboardtex.target)
        glBindTexture(self.scoreboardtex.target, self.scoreboardtex.id)
        t = self.scoreboardtex.tex_coords
        glBegin(GL_QUADS)
        glTexCoord2f(t[0], t[1])
        glVertex3f(x1, y2 - self.scoreboardimg.height, 0.0)
        glTexCoord2f(t[3], t[4])
        glVertex3f(x1 + self.scoreboardimg.width,
                   y2 - self.scoreboardimg.height, 0)
        glTexCoord2f(t[6], t[7])
        glVertex3f(x1 + self.scoreboardimg.width, y2, 0.0)
        glTexCoord2f(t[9], t[10])
        glVertex3f(x1, y2, 0.0)
        glEnd()

        x = x1 + self.scoreboardimg.width
        y = y2
        numclocks = int(self.g.scene.timeleft / Scoreboard.sec_per_clock)
        if numclocks < self.last_numclocks:
            self.g.sounddict['beeep'].play()
            self.last_numclocks = numclocks
        t = self.clocktex.tex_coords
        glBindTexture(self.clocktex.target, self.clocktex.id)
        glBegin(GL_QUADS)
        for i in range(numclocks + 1):
            glTexCoord2f(t[0], t[1])
            glVertex3f(x, y - self.clockimg.height, 0.0)
            glTexCoord2f(t[3], t[4])
            glVertex3f(x + self.clockimg.width, y - self.clockimg.height, 0)
            glTexCoord2f(t[6], t[7])
            glVertex3f(x + self.clockimg.width, y, 0.0)
            glTexCoord2f(t[9], t[10])
            glVertex3f(x, y, 0.0)
            x += self.clockimg.width
        glEnd()

        x = x1 + self.scoreboardimg.width
        y = y2 - 35.0
        numheads = self.g.scene.score
        t = self.enemyheadtex.tex_coords
        glBindTexture(self.enemyheadtex.target, self.enemyheadtex.id)
        glBegin(GL_QUADS)
        for i in range(numheads):
            glTexCoord2f(t[0], t[1])
            glVertex3f(x, y - self.enemyheadimg.height, 0.0)
            glTexCoord2f(t[3], t[4])
            glVertex3f(x + self.enemyheadimg.width,
                       y - self.enemyheadimg.height, 0)
            glTexCoord2f(t[6], t[7])
            glVertex3f(x + self.enemyheadimg.width, y, 0.0)
            glTexCoord2f(t[9], t[10])
            glVertex3f(x, y, 0.0)
            x += self.enemyheadimg.width
        glEnd()


