"""Test scene"""

import camera
import pyglet
from pyglet.gl import *
import mobsprite
import misc
import math
import random
import yaml
from pyglet.window import key
import imagedict

class Testscene(object):
    
    def __init__(self,g):
        self.g = g

#
# EDIT THIS LIST TO CHANGE THE MOBS YOU CAN PLACE
# each entry is the image name, lower case, without any extension
#
        self.mobtypes = ['fence', 'flowerbed', 'gate', 'gnome', 'topiary',
                         'tree', 'chair', 'george']

#
#
#

        self.gamefield_size = 2048.0
        
        self.camera = camera.Camera()
        self.camera.xrot= -60.0
        self.camera.yrot= 0.0
        self.camera.xpos = 1024.0
        self.camera.ypos = -370.0
        self.camera.zoom = -500.0

        self.autofollow = False

        self.grass_size = 8192.0
        self.grass = imagedict.grassimg()
        self.grass_texture = self.grass.get_texture()
        self.grass_scalewidth = self.grass_size / float(self.grass.width)
        self.grass_scaleheight = self.grass_size / float(self.grass.height)

        self.reticule = g.imagedict['reticule']
        self.reticule_tex = self.reticule.get_texture()
        self.rx, self.ry, self.rz = 0.0, 0.0, 0.0
        self.last_rx, self.last_ry, self.last_rz = 0.0, 0.0, 0.0

        self.mx = 0.0
        self.my = 0.0
        self.mouse_moved = False

        self.mouse_scale = 4.0
        self.move_step = 200.0
        self.mobmove = False
        self.scene_name = 'testscene'


        self.moblist = []
        self.fencehack = mobsprite.Mobsprite(self.g, 0, 0, 'fence')
        self.gatehack = mobsprite.Mobsprite(self.g, 0, 0, 'gate')
        self.porchhack = mobsprite.Mobsprite(self.g, 0, 0, 'porch')

        #self.level_load();
        #return
        #for i in range(10):
        #    self.moblist.append( \
        #        mobsprite.Mobsprite(random.random() * self.gamefield_size,
        #                            random.random() * self.gamefield_size,
        #                            mobtypes[int(random.random() * 4)]))
        #self.level_save()

        print "Test scene initialized!  Press h for help."
        

    # simulation step
    def run(self):
        pass


    def level_save(self):
        dumplist = []
        for mob in self.moblist:
            dumplist.append(mob.dump())
        print yaml.dump(dumplist)
        configfile = open(misc.configpath(self.scene_name + '.yaml'), 'w')
        yaml.dump(dumplist, configfile)
        configfile.close()
        print "Saved scene to " + self.scene_name


    def level_load(self):
        configfile = open(misc.configpath(self.scene_name + '.yaml'), 'r')
        moblist = yaml.load(configfile)
        configfile.close()
        self.moblist = []
        for mobinfo in moblist:
            self.moblist.append(mobsprite.Mobsprite( \
                self.g, mobinfo['x'], mobinfo['y'], mobinfo['type']))
        print "Loaded scene from " + self.scene_name


    def on_mouse_drag(self, x, y, dx, dy, buttons, mods):
        self.autofollow = False
        if buttons & pyglet.window.mouse.LEFT:
            self.camera.rotate_cam(dx,dy)
        elif buttons & pyglet.window.mouse.RIGHT:
            self.camera.move_cam(dx * self.mouse_scale, dy * self.mouse_scale)
        self.on_mouse_motion(x, y, dx, dy)

    def on_mouse_motion(self, x, y, dx, dy):
        (self.mx, self.my) = (x, y)
        self.mouse_moved = True

    def on_mouse_scroll(self, x, y, scroll_x, scroll_y):
        self.camera.zoom_cam(scroll_y)

    def on_mouse_press(self, x, y, buttons, mods):
        pass

    # Keyboard controls
    def on_key_press(self, keysym, mod):
        if keysym == key.H:
            self.print_help()
        elif keysym == key.C:
            c = self.camera
            print "cam pos: x,y %d, %d   rot %d, %d  zoom: %d" \
                  % (c.xpos, c.ypos, c.xrot, c.yrot, c.zoom)
        elif keysym == key.Q:
            print "Quitting!"
            self.g.gamestate = 'quit'
        elif keysym == key.A:
            self.autofollow = not self.autofollow
        elif keysym == key.UP:
            self.camera.ypos += self.move_step
        elif keysym == key.DOWN:
            self.camera.ypos -= self.move_step
        elif keysym == key.LEFT:
            self.camera.xpos -= self.move_step
        elif keysym == key.RIGHT:
            self.camera.xpos += self.move_step
        elif int(keysym) >= int(key._1) and int(keysym) <= int(key._9):
            idx = int(keysym) - int(key._1)
            if idx < len(self.mobtypes):
                print "adding mob: %s" % self.mobtypes[idx]
                self.moblist.append( \
                mobsprite.Mobsprite(self.g, self.rx, self.ry,
                                    self.mobtypes[idx]))
        elif keysym == key.P:
            self.print_mobtypes()
        elif keysym == key.S:
            self.level_save()
        elif keysym == key.L:
            self.level_load()
        elif keysym == key.D:
            self.delete_mob()
        elif keysym == key.M:
            self.mobmove = not self.mobmove
            

    def on_draw(self):
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        glLoadIdentity()
        glRotatef(self.camera.xrot,1.0,0.0,0.0);
        glRotatef(self.camera.yrot,0.0,0.0,1.0);
        glTranslatef(-self.camera.xpos, -self.camera.ypos, self.camera.zoom)

        # Calculate position of reticule
        (self.rx, self.ry, self.rz) = misc.screen_to_world(self.mx, self.my)
        # Follow reticule if autofollow is active and the mouse was moved
        if self.autofollow == True and self.mouse_moved == True:
            dx, dy, dz = self.rx - self.last_rx, self.ry - self.last_ry, \
                         self.rz - self.last_rz
            glTranslatef(-dx, -dy, -dz)
            self.camera.xpos += dx
            self.camera.ypos += dy
            self.camera.zoom += dz
            self.mouse_moved = False
        # Move last mob to reticule, if active
        if self.mobmove == True and len(self.moblist) > 0:
            mob = self.moblist[len(self.moblist) - 1]
            mob._x = self.rx
            mob._y = self.ry
        self.last_rx, self.last_ry, self.last_rz = self.rx, self.ry, self.rz

        glEnable(self.grass_texture.target)
        glBindTexture(self.grass_texture.target, self.grass_texture.id)
        t = self.grass_texture.tex_coords
        glBegin(GL_QUADS);
        glTexCoord2f(t[0] * self.grass_scalewidth,
                     t[1] * self.grass_scaleheight);
        glVertex3f(-self.grass_size, -self.grass_size,  0.0);
        glTexCoord2f(t[3] * self.grass_scalewidth,
                     t[4] * self.grass_scaleheight);
        glVertex3f( self.grass_size, -self.grass_size,  0.0);
        glTexCoord2f(t[6] * self.grass_scalewidth,
                     t[7] * self.grass_scaleheight);
        glVertex3f( self.grass_size,  self.grass_size,  0.0);
        glTexCoord2f(t[9] * self.grass_scalewidth,
                     t[10] * self.grass_scaleheight);
        glVertex3f(-self.grass_size,  self.grass_size, 0.0);
        glEnd();

        glLineWidth(5.0)
        glBegin(GL_LINE_LOOP)
        glVertex3f(0.0, 0.0, 0.0)
        glVertex3f(self.gamefield_size, 0.0, 0.0)
        glVertex3f(self.gamefield_size, self.gamefield_size, 0.0)
        glVertex3f(0.0, self.gamefield_size, 0.0)
        glEnd()

        glEnable(self.reticule_tex.target)
        glBindTexture(self.reticule_tex.target, self.reticule_tex.id)
        t = self.reticule_tex.tex_coords
        glBegin(GL_QUADS)
        glTexCoord2f(t[0],t[1])
        glVertex3f(self.rx - 64.0, self.ry - 64.0, self.rz)
        glTexCoord2f(t[3],t[4])
        glVertex3f(self.rx + 64.0, self.ry - 64.0, self.rz)
        glTexCoord2f(t[6],t[7])
        glVertex3f(self.rx + 64.0, self.ry + 64.0, self.rz)
        glTexCoord2f(t[9],t[10])
        glVertex3f(self.rx - 64.0, self.ry + 64.0, self.rz)
        glEnd()

        self.draw_fence()

        for mob in sorted(self.moblist, lambda a,b: cmp(b._y, a._y)):
            mob.draw()


    def print_help(self):
        print "A - Put camera in auto-follow mode (flaky!)"
        print "Arrow keys - Move camera"
        print "C - Prints the camera position"
        print "D - Deletes last mob on list"
        print "H - This help screen"
        print "L - Load scene from yaml file '%s'" % self.scene_name
        print "M - Move last mob on list"
        print "P - Print a list of available mobs"
        print "S - Save scene to yaml file '%s'" % self.scene_name
        print "Q or ESC - Quits game"


    def print_mobtypes(self):
        idx = 1
        for mob in self.mobtypes:
            print "%d - %s" % (idx, mob)
            idx += 1

    def delete_mob(self):
        if len(self.moblist) > 0:
            mob = self.moblist.pop()
            print "Deleted %s" % mob.type
        else:
            print "Mob list is empty!"


    # Draws fence and the gate and the porch
    def draw_fence(self):
        # back fence with gate
        self.fencehack.rot = 0
        self.fencehack._y = 2048.0
        x = self.fencehack.image.width / 2.0
        for i in range(4):
            self.fencehack._x = x
            self.fencehack.draw()
            x += self.fencehack.image.width
        self.gatehack._y = 2048.0
        self.gatehack._x = x - self.fencehack.image.width / 2.0 \
                           + self.gatehack.image.width / 2.0
        self.gatehack.draw()
        x = self.gatehack._x + self.gatehack.image.width / 2.0 \
            + self.fencehack.image.width / 2.0
        for i in range(4):
            self.fencehack._x = x
            self.fencehack.draw()
            x += self.fencehack.image.width
        # side fences
        rightx = x - self.fencehack.image.width / 2.0
        self.fencehack.rot = 1
        y = 2048.0 - self.fencehack.image.width / 2.0
        for i in range(8):
            self.fencehack._y = y
            self.fencehack._x = 0
            self.fencehack.draw()
            self.fencehack._x = rightx
            self.fencehack.draw()
            y -= self.fencehack.image.width
        # porch
        self.porchhack.rot = 3
        self.porchhack._y = -self.porchhack.image.height
        x = self.porchhack.image.width / 2.0
        for i in range(11):
            self.porchhack._x = x
            self.porchhack.draw()
            x += self.porchhack.image.width


        
        


        
